%% Matlab file to calculate and plot the
% Drucker-Prager failure surface
% written by Gertraud Medicus within the project
% 'Animating Soil Models' - 01/2021
% University of Innsbruck, Austria
% -------------------------------------------------------------------
% useful information is in e.g. Griffiths (1990):
% 'Failure Criteria Interpretation Based on Mohr Coulomb Friction'
% https://doi.org/10.1061/(ASCE)0733-9410(1990)116:6(986)
% and Griffiths & Huang (2009):
% https://doi.org/10.1002/nag.810
% -------------------------------------------------------------------

close all
clear all
clc

disp('================================================')
disp('                                                ')
disp('       Drucker-Prager failure surface           ')
disp('           for phi=phi_c and c=0                ')
disp('          Gertraud Medicus (2021)               ')
disp('     https://soilmodels.com/soilanim/           ')
disp('                                                ')
disp('================================================')

%% choose friction angle
phi = 30*pi/180; % in radian
%% f=0 according to Drucker-Prager
Theta = -180:180;
j=1;
for p=[1 100*sqrt(3)] % mean effective stress
    for i = Theta
        theta = i*pi/180;
        s=3*p/sqrt(3); % invariant (I) s, eq. 5
        
        % Drucker-Prager, see e.g. Griffiths (1990) eq. 14
        L=2*sqrt(2)*sin(phi)/(3-sin(phi));
        t=L*s;
        
        theta = i*pi/180;
        %% e.g. Griffiths & Huang (2009) eq. (11)
        
        sigma(j,1) =  s/sqrt(3)+sqrt(2/3)*t*sin(theta-2*pi/3);
        sigma(j,2) = s/sqrt(3)+sqrt(2/3)*t*sin(theta);
        sigma(j,3) = s/sqrt(3)+sqrt(2/3)*t*sin(theta+2*pi/3);
        
        j=j+1;
    end
    
end

%% reshape coordinates for 3D plot
N=2;
siz = size(sigma(:,1)',2)/N;
x = reshape(sigma(:,2)',siz,N);
y = reshape(sigma(:,1)',siz,N);
z = reshape(sigma(:,3)',siz,N);
%% 3D plot
figure(1)
hold on;
set(gcf,'defaultaxesfontsize',10);
set(gcf,'defaulttextfontsize',10);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperSize', [8.3 8.3]);
set(gcf, 'PaperPosition', [0 0 8.3 8.3]);
cone_clr=[.2 .5 .2]; % color cone
line_clr=[.9 .5 0]; % color line
iso_clr=[0 0.2 .4]; % color iso ax
colormap(cone_clr) % colormap

%% 3D surface, facealpha to control transparency
colormap(cone_clr) % colormap cone
surf(x,y,z,'FaceAlpha',1); hold on
plot3(x(end-360:end),y(end-360:end),z(end-360:end),'-','Color',line_clr); hold on
%plot3(x,y,z,'-','Color',line_clr); hold on
%% hydrostatic axis
iso=[0,0,0;350,350,350]';
plot3(iso(1,:),iso(2,:),iso(3,:),'-','Color',iso_clr,'Linewidth',.5)
%% axes
plot3([-10 350],[0 0],[0 0],'-','Color','k','Linewidth',.5);
plot3([0 0],[-10 350],[0 0],'-','Color','k','Linewidth',.5);
plot3([0 0],[0 0],[-10 350],'-','Color','k','Linewidth',.5);
text(0,0,360,'$$\sigma_1''$$', 'Interpreter','Latex');hold on
text(360,0,0,'$$\sigma_2''$$', 'Interpreter','Latex');hold on
text(0,360,0,'$$\sigma_3''$$', 'Interpreter','Latex');hold on

%% or use default Matlab axes
% Ticks = 0:500:400;
% set(gca, 'XTickMode', 'manual', 'XTick', Ticks, 'xlim', [0 360]);
% set(gca, 'YTickMode', 'manual', 'YTick', Ticks, 'ylim', [0 360]);
% set(gca, 'ZTickMode', 'manual', 'ZTick', Ticks, 'zlim', [0 360]);
% xlabel('$$\sigma_2''$$', 'Interpreter','Latex');
% ylabel('$$\sigma_3''$$', 'Interpreter','Latex');
% zlabel('$$\sigma_1''$$', 'Interpreter','Latex');

axis equal; grid off; axis off;
set(gca,'LooseInset',get(gca,'TightInset')); % remove border around figure
%
xlim([-inf inf]);
ylim([-inf inf]);
zlim([-inf inf]);

view([80 15]);
% view([135 atand(1/sqrt(2))]); % view deviatoric plane

%% light settings
camlight('left')
shading interp
camlight(-30,-atand(1/sqrt(2)))
camlight(30,-atand(1/sqrt(2)))
set(gcf,'Renderer','zbuffer')
set(findobj(gca,'type','surface'),...
    'FaceLighting','phong',...
    'AmbientStrength',.3,'DiffuseStrength',.8,...
    'SpecularStrength',.9,'SpecularExponent',25,...
    'BackFaceLighting','unlit')

% print figure
print(gcf,'-r300','-dpng','DP.png')